#!/sbin/sh
#
###########################################
#
# Copyright (C) 2020 FlameGApps Project
#
# This file is part of the FlameGApps Project by ayandebnath <ayandebnath000@gmail.com>
#
# The FlameGApps scripts are free software, you can redistribute it and/or modify it
# 
# These scripts are distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY
#
###########################################
# File Name    : installer.sh
# Last Updated : 2020-09-15
###########################################
##
# List of the basic edition gapps files
basic_gapps_list="
CalendarSync
DigitalWellbeing
GoogleLocationHistory
MarkupGoogle
SetupWizard
SoundPickerGoogle"

# List of the full edition gapps files
full_gapps_list="
CalendarSync
DeviceHealthServices
DigitalWellbeing
GoogleClock
GoogleCalendar
GoogleCalculator
GoogleContacts
GoogleDialer
GoogleMessages
GoogleKeyboard
GooglePhotos
GoogleLocationHistory
MarkupGoogle
SetupWizard
SoundPickerGoogle
WallpaperPickerGoogle"

# Pre-installed unnecessary app list
rm_list_basic="
app/ExtShared
app/FaceLock
app/GoogleExtShared
app/GoogleContactSyncAdapter
priv-app/ExtServices
priv-app/AndroidPlatformServices
priv-app/GoogleServicesFramework
priv-app/GmsCoreSetupPrebuilt
priv-app/GmsCore
priv-app/PrebuiltGmsCore
priv-app/PrebuiltGmsCorePi
priv-app/PrebuiltGmsCoreQt
priv-app/Phonesky
priv-app/SetupWizard
priv-app/Wellbeing
priv-app/wellbeing
priv-app/WellbeingGooglePrebuilt
priv-app/WellbeingPrebuilt"

rm_list_full="
app/AudioFX
app/ExtShared
app/Etar
app/FaceLock
app/Clock
app/DeskClock
app/DashClock
app/PrebuiltDeskClock
app/Calculator
app/Calculator2
app/ExactCalculator
app/Calendar
app/CalendarPrebuilt
app/Eleven
app/message
app/messages
app/Messages
app/Markup
app/MarkupGoogle
app/PrebuiltBugle
app/Hangouts
app/SoundPicker
app/PrebuiltSoundPicker
app/SoundPickerPrebuilt
app/Contact
app/Contacts
app/ChromePublic
app/Photos
app/PhotosPrebuilt
app/SimpleGallery
app/GalleryGoPrebuilt
app/CalculatorGooglePrebuilt
app/CalendarGooglePrebuilt
app/Messaging
app/Messenger
app/messaging
app/RevengeMessages
app/Email
app/Email2
app/Gmail
app/Maps
app/LatinIMEGooglePrebuilt
app/Browser
app/Browser2
app/Jelly
app/Via
app/ViaBrowser
app/LatinIME
app/LatinIMEPrebuilt
priv-app/AudioFX
priv-app/ExtServices
priv-app/Browser
priv-app/Browser2
priv-app/Jelly
priv-app/Via
priv-app/ViaBrowser
priv-app/LatinIME
priv-app/GoogleServicesFramework
priv-app/PrebuiltGmsCore
priv-app/PrebuiltGmsCorePi
priv-app/GmsCore
priv-app/SetupWizard
priv-app/SetupWizardPrebuilt
priv-app/PixelSetupWizard
priv-app/Wellbeing
priv-app/CarrierSetup
priv-app/ConfigUpdater
priv-app/GmsCoreSetupPrebuilt
priv-app/Gallery
priv-app/Gallery2
priv-app/Gallery3d
priv-app/GalleryGo
priv-app/GalleryGoPrebuilt
priv-app/PrebuiltGalleryGo
priv-app/SimpleGallery
priv-app/Camera2
priv-app/Photos
priv-app/Contact
priv-app/Contacts
priv-app/GoogleContacts
priv-app/GoogleDialer
priv-app/Music
priv-app/Music2
priv-app/RetroMusic
priv-app/crDroidMusic
priv-app/SnapGallery
priv-app/SnapdragonGallery
priv-app/Clock
priv-app/Calendar
priv-app/Calculator
priv-app/Hangouts
priv-app/Messaging
priv-app/Gmail
priv-app/Email
priv-app/Email2
priv-app/Eleven
priv-app/Maps
priv-app/GoogleMaps
priv-app/SounPicker
priv-app/GoogleMapsPrebuilt
priv-app/MarkupGoogle
priv-app/PrebuiltDeskClock
priv-app/SoundPickerPrebuilt
priv-app/PrebuiltSoundPicker
priv-app/Turbo
priv-app/Wallpaper
priv-app/Wallpapers
priv-app/WallpaperPrebuilt
priv-app/WallpapersPrebuilt
priv-app/WallpapersGooglePrebuilt
priv-app/WallpaperGooglePrebuilt
priv-app/DeviceHealthService
priv-app/AndroidPlafoPlatformServices
priv-app/LatinIMEGooglePrebuilt"

ui_print() {
  echo "ui_print $1
    ui_print" >> $OUTFD
}

set_progress() { echo "set_progress $1" >> $OUTFD; }

is_mounted() { mount | grep -q " $1 "; }

recovery_actions() {
  OLD_LD_LIB=$LD_LIBRARY_PATH
  OLD_LD_PRE=$LD_PRELOAD
  OLD_LD_CFG=$LD_CONFIG_FILE
  unset LD_LIBRARY_PATH
  unset LD_PRELOAD
  unset LD_CONFIG_FILE
}

recovery_cleanup() {
  [ -z $OLD_LD_LIB ] || export LD_LIBRARY_PATH=$OLD_LD_LIB
  [ -z $OLD_LD_PRE ] || export LD_PRELOAD=$OLD_LD_PRE
  [ -z $OLD_LD_CFG ] || export LD_CONFIG_FILE=$OLD_LD_CFG
}

unmount_all() {
  ui_print " "
  ui_print "- Unmounting partitions"
  for m in "/system" "/system_root" "/vendor" "/product"; do
    if [ -e $m ]; then
      (umount $m
      umount -l $m) 2>/dev/null
    fi
  done
}

clean_up() {
  rm -rf /tmp/flamegapps
  rm -rf /tmp/config.prop
  rm -rf /tmp/flame.prop
  rm -rf /tmp/addon.d.sh
  rm -rf /tmp/tar_gapps
  rm -rf /tmp/unzip_dir
}

path_info() {
  ls / > $root_info
  ls -R $SYSTEM > $system_info
}

abort() {
  sleep 1
  ui_print "- Aborting..."
  sleep 3
  path_info
  unmount_all
  take_logs
  clean_up
  recovery_cleanup
  exit 1;
}

exit_all() {
  sleep 0.5
  path_info
  unmount_all
  sleep 0.5
  set_progress 0.90
  take_logs
  clean_up
  recovery_cleanup
  sleep 0.5
  ui_print " "
  ui_print "- Installation Successful..!"
  ui_print " "
  set_progress 1.00
  exit 0;
}

# Unmount before mounting
unmount_all

# Mount partitions
set_progress 0.10
ui_print "- Mounting partitions"
sleep 1
dynamic_partitions=`getprop ro.boot.dynamic_partitions`
slot=`getprop ro.boot.slot_suffix`
[ ! -z "$slot" ] && ui_print "- Current boot slot: $slot"

if [ -n "$(cat /etc/fstab | grep /system_root)" ]; then
  MOUNT_POINT=/system_root
else
  MOUNT_POINT=/system
fi

for p in "/cache" "/data" "$MOUNT_POINT" "/vendor"; do
  if [ -d "$p" ] && grep -q "$p" "/etc/fstab" && ! is_mounted "$p"; then
    mount "$p"
  fi
done

if [ "$dynamic_partitions" = "true" ]; then
  ui_print "- Dynamic partition detected"
  mount /dev/block/mapper/system$slot /system_root
  mount /dev/block/mapper/system$slot /vendor 2>/dev/null
  mount /dev/block/mapper/system$slot /product 2>/dev/null
else
  mount /dev/block/bootdevice/by-name/system$slot /system_root 2>/dev/null
fi

if [ "$dynamic_partitions" = "true" ]; then
  for block in system vendor product; do
    for slot in "" _a _b; do
      blockdev --setrw /dev/block/mapper/$block$slot 2>/dev/null
    done
  done
fi

mount -o rw,remount $MOUNT_POINT

sleep 0.3

if is_mounted /system_root; then
  ui_print "- Device is system-as-root"
  if [ -f /system_root/build.prop ]; then
    mount -o bind /system_root /system
    SYSTEM=/system_root
    ui_print "- System is $SYSTEM"
  else
    mount -o bind /system_root/system /system
    SYSTEM=/system_root/system
    ui_print "- System is $SYSTEM"
  fi
elif is_mounted /system; then
  if [ -f /system/build.prop ]; then
    SYSTEM=/system
    ui_print "- system is $SYSTEM"
  elif [ -f /system/system/build.prop ]; then
    ui_print "- Device is system-as-root"
    mkdir -p /system_root
    mount --move /system /system_root
    mount -o bind /system_root/system /system
    SYSTEM=/system_root/system
    ui_print "- System is /system/system"
  fi
else
  ui_print "- Failed to mount/detect system"
  abort
fi

ui_print " "

recovery_actions

PROPFILES="$SYSTEM/build.prop $TMP/flame.prop"

get_file_prop() {
  grep -m1 "^$2=" "$1" | cut -d= -f2
}

get_prop() {
  #check known .prop files using get_file_prop
  for f in $PROPFILES; do
    if [ -e "$f" ]; then
      prop="$(get_file_prop "$f" "$1")"
      if [ -n "$prop" ]; then
        break #if an entry has been found, break out of the loop
      fi
    fi
  done
  #if prop is still empty; try to use recovery's built-in getprop method; otherwise output current result
  if [ -z "$prop" ]; then
    getprop "$1" | cut -c1-
  else
    printf "$prop"
  fi
}

log_folder="$(dirname "$ZIPFILE")"
mkdir -p $TMP/flamegapps/logs
log_dir="$TMP/flamegapps/logs"
flame_log="$log_dir/installation_log.txt"
flame_prop="$TMP/flame.prop"
build_prop="$SYSTEM/build.prop"
build_info="$log_dir/build_info.prop"
root_info="$log_dir/rootpathinfo.txt"
system_info="$log_dir/systempathinfo.txt"

take_logs() {
  ui_print " "
  ui_print "- Copying logs to $log_folder"
  cp -f $TMP/recovery.log $log_dir/recovery.log
  cd $log_dir
  tar -cz -f "$TMP/flamegapps_debug_logs.tar.gz" *
  cp -f $TMP/flamegapps_debug_logs.tar.gz $log_folder/flamegapps_debug_logs.tar.gz
  cd /
  rm -rf $TMP/flamegapps_debug_logs.tar.gz
}

# Get ROM, device & package information
flame_android=`get_prop ro.flame.android`
flame_sdk=`get_prop ro.flame.sdk`
flame_arch=`get_prop ro.flame.arch`
flame_edition=`get_prop ro.flame.edition`
rom_version=`get_prop ro.build.version.release`
rom_sdk=`get_prop ro.build.version.sdk`
device_architecture=`get_prop ro.product.cpu.abilist`
device_code=`get_prop ro.product.device`

if [ -z "$device_architecture" ]; then
  device_architecture=`get_prop ro.product.cpu.abi`
fi

case "$device_architecture" in
  *x86_64*) arch="x86_64"
    ;;
  *x86*) arch="x86"
    ;;
  *arm64*) arch="arm64"
    ;;
  *armeabi*) arch="arm"
    ;;
  *) arch="unknown"
    ;;
esac

echo --------------------------------------------------------------- >> $flame_log
echo "---------- FlameGApps Installation Logs ----------" >> $flame_log
echo "  " >> $flame_log
echo "- Mount Point: $MOUNT_POINT" >> $flame_log
echo "- Current slot: $active_slot" >> $flame_log
echo "- Dynamic partition: $dynamic_partitions" >> $flame_log
echo "- Flame version: $flame_android" >> $flame_log
echo "- Flame SDK: $flame_sdk" >> $flame_log
echo "- Flame ARCH: $flame_arch" >> $flame_log
echo "- ROM version: $rom_version" >> $flame_log
echo "- ROM SDK: $rom_sdk" >> $flame_log
echo "- Device ARCH: $device_architecture ($arch)" >> $flame_log
echo "- Device code: $device_code" >> $flame_log

# Get prop details before compatibility checks
cat $build_prop >> $build_info
cat $flame_prop >> $build_info

set_progress 0.20
sleep 1
ui_print "- Android: $rom_version, SDK: $rom_sdk, ARCH: $arch"
sleep 1

if [ ! "$rom_sdk" = "$flame_sdk" ]; then
  ui_print " "
  ui_print "****************** WARNING ******************"
  ui_print " "
  ui_print "! Wrong android version detected"
  sleep 0.5
  ui_print "! This package is for android: $flame_android only"
  sleep 0.5
  ui_print "Your ROM is Android: $rom_version"
  sleep 0.5
  ui_print " "
  ui_print "******* FlameGApps Installation Failed *******"
  ui_print " "
  abort
fi

if [ ! "$arch" = "$flame_arch" ]; then
  ui_print " "
  ui_print "****************** WARNING ******************"
  ui_print " "
  ui_print "! Wrong device architecture detected"
  sleep 0.5
  ui_print "This package is for device: $flame_arch onl "
  sleep 0.5
  ui_print "Your device is: $arch"
  sleep 0.5
  ui_print " "
  ui_print "******* FlameGApps Installation Failed *******"
  ui_print " "
  abort
fi

# Remove pre-installed unnecessary system apps
ui_print " "
if [ "$flame_edition" = "basic" ]; then
  ui_print "- Removing unnecessary system apps"
  set_progress 0.30
  sleep 0.5
  echo "  " >> $flame_log
  echo "- Removing basic list files" >> $flame_log
  for f in $rm_list_basic; do
    rm -rf $SYSTEM/$f
    [ "$flame_sdk" -gt "28" ] && rm -rf $SYSTEM/product/$f
    [ "$flame_edition" -gt "29" ] && rm -rf $SYSTEM/system_ext/$f
  done
elif [ "$flame_edition" = "full" ]; then
  ui_print "- Removing unnecessary system apps"
  set_progress 0.30
  sleep 0.5
  echo "  " >> $flame_log
  echo "- Removing full list files" >> $flame_log
  for f in $rm_list_full; do
    rm -rf $SYSTEM/$f
    [ "$flame_sdk" -gt "28" ] && rm -rf $SYSTEM/product/$f
    [ "$flame_edition" -gt "29" ] && rm -rf $SYSTEM/system_ext/$f
  done
else
  ui_print "****************** WARNING *******************"
  ui_print " "
  sleep 0.5
  echo "- Failed to detect edition type" >> $flame_log
  ui_print "! Failed to detect FlameGApps edition type"
  sleep 0.5
  ui_print " "
  ui_print "******* FlameGApps Installation Failed *******"
  abort
fi

CORE_DIR="$TMP/tar_core"
GAPPS_DIR="$TMP/tar_gapps"
UNZIP_FOLDER="$TMP/unzip_dir"
EX_SYSTEM="$UNZIP_FOLDER/system"

ui_print " "
ui_print "- Creating unzip directory in tmp"
sleep 0.5
mkdir $TMP/unzip_dir

if [ "$flame_edition" = "basic" ]; then
  gapps_list="$basic_gapps_list"
elif [ "$flame_edition" = "full" ]; then
  gapps_list="$full_gapps_list"
fi

unzip_core() {
  set_progress 0.40;
  ui_print " "
  ui_print "- Copying core files to unzip directory"
  ui_print " "
  unzip -o "$ZIPFILE" 'tar_core/*' -d $TMP
}

unzip_gapps() {
  set_progress 0.60;
  ui_print " "
  ui_print "- Copying gapps files to unzip directory"
  ui_print " "
  unzip -o "$ZIPFILE" 'tar_gapps/*' -d $TMP
}

install_core() {
  set_progress 0.50
  ui_print "- Installing Core GApps"
  tar -xf "$CORE_DIR/Core.tar.xz" -C $UNZIP_FOLDER
  file_list="$(find "$EX_SYSTEM/" -mindepth 1 -type f | cut -d/ -f5-)"
  dir_list="$(find "$EX_SYSTEM/" -mindepth 1 -type d | cut -d/ -f5-)"
  for file in $file_list; do
    install -D "$EX_SYSTEM/${file}" "$SYSTEM/${file}"
    chcon -h u:object_r:system_file:s0 "$SYSTEM/${file}"
    chmod 0644 "$SYSTEM/${file}"
  done
  for dir in $dir_list; do
    chcon -h u:object_r:system_file:s0 "$SYSTEM/${dir}"
    chmod 0755 "$SYSTEM/${dir}"
  done
  rm -rf $CORE_DIR
  rm -rf $UNZIP_FOLDER/system
}

install_gapps() {
  set_progress 0.70
  for g in $gapps_list; do
    ui_print "- Installing $g"
    tar -xf "$GAPPS_DIR/$g.tar.xz" -C $UNZIP_FOLDER
    rm -rf $GAPPS_DIR/$g.tar.xz
  done
  file_list="$(find "$EX_SYSTEM/" -mindepth 1 -type f | cut -d/ -f5-)"
  dir_list="$(find "$EX_SYSTEM/" -mindepth 1 -type d | cut -d/ -f5-)"
  for file in $file_list; do
    install -D "$EX_SYSTEM/${file}" "$SYSTEM/${file}"
    chcon -h u:object_r:system_file:s0 "$SYSTEM/${file}"
    chmod 0644 "$SYSTEM/${file}"
  done
  for dir in $dir_list; do
    chcon -h u:object_r:system_file:s0 "$SYSTEM/${dir}"
    chmod 0755 "$SYSTEM/${dir}"
  done
  rm -rf $UNZIP_FOLDER/system
}

# remove_line <file> <line match string>
# By osm0sis @xda-developers
# For reference, check https://github.com/osm0sis/AnyKernel3/blob/master/tools/ak3-core.sh
remove_line() {
  if grep -q "$2" $1; then
    local line=$(grep -n "$2" $1 | head -n1 | cut -d: -f1);
    sed -i "${line}d" $1;
  fi
}

run_basic_config() {
  set_progress 0.75
  if [ -f $log_folder/flamegapps-config.txt ]; then
    cp -f $log_folder/flamegapps-config.txt $TMP/config.prop
    cp -f $TMP/config.prop $log_dir/config.prop
    ui_print " "
    ui_print "- Config detected in $log_folder"
    ui_print " "
    sleep 1
    rm_setupwizard=`get_file_prop $TMP/config.prop ro.basic.remove.setupwizard`
    rm_wellbeing=`get_file_prop $TMP/config.prop ro.basic.remove.wellbeing`
    rm_sounds=`get_file_prop $TMP/config.prop ro.basic.remove.sounds`
    rm_markup=`get_file_prop $TMP/config.prop ro.basic.remove.markup`
    if [ "$rm_setupwizard" = "1" ]; then
      ui_print "- Removing SetupWizard"
      sleep 0.3
      rm -rf $SYSTEM/priv-app/GoogleRestore
      rm -rf $SYSTEM/priv-app/GoogleBackupTransport
      rm -rf $SYSTEM/priv-app/AndroidMigratePrebuilt
      rm -rf $SYSTEM/priv-app/SetupWizard
      remove_line $TMP/addon.d.sh "priv-app/GoogleRestore/GoogleRestore.apk"
      remove_line $TMP/addon.d.sh "priv-app/GoogleBackupTransport/GoogleBackupTransport.apk"
      remove_line $TMP/addon.d.sh "priv-app/SetupWizard/SetupWizard.apk"
      remove_line $TMP/addon.d.sh "priv-app/AndroidMigratePrebuilt/AndroidMigratePrebuilt.apk"
    fi
    if [ "$rm_sounds" = "1" ]; then
      ui_print "- Removing GoogleSoundPicker"
      sleep 0.3
      rm -rf $SYSTEM/app/SoundPickerPrebuilt
      remove_line $TMP/addon.d.sh "app/SoundPickerPrebuilt/SoundPickerPrebuilt.apk"
    fi
    if [ "$rm_markup" = "1" ]; then
      ui_print "- Removing GoogleMarkup"
      sleep 0.3
      rm -rf $SYSTEM/app/MarkupGoogle
      rm -rf $SYSTEM/app/lib64/libsketchology_native.so
      remove_line $TMP/addon.d.sh "app/MarkupGoogle/MarkupGoogle.apk"
      remove_line $TMP/addon.d.sh "lib64/libsketchology_native.so"
      remove_line $TMP/addon.d.sh "app/MarkupGoogle/lib/arm64/libsketchology_native.so"
    fi
    if [ "$rm_wellbeing" = "1" ]; then
      ui_print "- Removing DigitalWellbeing"
      sleep 0.3
      rm -rf $SYSTEM/priv-app/WellbeingPrebuilt
      remove_line $TMP/addon.d.sh "priv-app/WellbeingPrebuilt/WellbeingPrebuilt.apk"
    fi
  fi
}

run_full_config() {
  set_progress 0.75
  if [ -f $log_folder/flamegapps-config.txt ]; then
    cp -f $log_folder/flamegapps-config.txt $TMP/config.prop
    cp -f $TMP/config.prop $log_dir/config.prop
    ui_print " "
    ui_print "- Config detected in $log_folder"
    ui_print " "
    sleep 1
    rm_setupwizard=`get_file_prop $TMP/config.prop ro.full.remove.setupwizard`
    rm_wellbeing=`get_file_prop $TMP/config.prop ro.full.remove.wellbeing`
    rm_photos=`get_file_prop $TMP/config.prop ro.full.remove.photos`
    rm_calendar=`get_file_prop $TMP/config.prop ro.full.remove.calendar`
    rm_calculator=`get_file_prop $TMP/config.prop ro.full.remove.calculator`
    rm_sounds=`get_file_prop $TMP/config.prop ro.full.remove.sounds`
    rm_wallpapers=`get_file_prop $TMP/config.prop ro.full.remove.wallpapers`
    rm_gdialer=`get_file_prop $TMP/config.prop ro.full.remove.gdialer`
    rm_markup=`get_file_prop $TMP/config.prop ro.full.remove.markup`
    keep_snap=`get_file_prop $TMP/config.prop ro.full.keep.snap`
    if [ "$rm_setupwizard" = "1" ]; then
      ui_print "- Removing SetupWizard"
      sleep 0.3
      rm -rf $SYSTEM/priv-app/GoogleRestore
      rm -rf $SYSTEM/priv-app/GoogleBackupTransport
      rm -rf $SYSTEM/priv-app/AndroidMigratePrebuilt
      rm -rf $SYSTEM/priv-app/SetupWizard
      remove_line $TMP/addon.d.sh "priv-app/GoogleRestore/GoogleRestore.apk"
      remove_line $TMP/addon.d.sh "priv-app/GoogleBackupTransport/GoogleBackupTransport.apk"
      remove_line $TMP/addon.d.sh "priv-app/SetupWizard/SetupWizard.apk"
      remove_line $TMP/addon.d.sh "priv-app/AndroidMigratePrebuilt/AndroidMigratePrebuilt.apk"
    fi
    if [ "$rm_wellbeing" = "1" ]; then
      ui_print "- Removing DigitalWellbeing"
      sleep 0.3
      rm -rf $SYSTEM/priv-app/WellbeingPrebuilt
      remove_line $TMP/addon.d.sh "priv-app/WellbeingPrebuilt/WellbeingPrebuilt.apk"
    fi
    if [ "$rm_gdialer" = "1" ]; then
      ui_print "- Removing GoogleDialer"
      sleep 0.3
      rm -rf $SYSTEM/priv-app/GoogleDialer
      remove_line $TMP/addon.d.sh "priv-app/GoogleDialer/GoogleDialer.apk"
      remove_line $TMP/addon.d.sh "priv-app/Dialer"
    fi
    if [ "$rm_photos" = "1" ]; then
      ui_print "- Removing GooglePhotos"
      sleep 0.3
      rm -rf $SYSTEM/app/Photos
      remove_line $TMP/addon.d.sh "app/Photos/Photos.apk"
    fi
    if [ "$rm_calendar" = "1" ]; then
      ui_print "- Removing GoogleCalendar"
      sleep 0.3
      rm -rf $SYSTEM/app/CalendarGooglePrebuilt
      remove_line $TMP/addon.d.sh "app/CalendarGooglePrebuilt/CalendarGooglePrebuilt.apk"
    fi
    if [ "$rm_calculator" = "1" ]; then
      ui_print "- Removing GoogleCalculator"
      sleep 0.3
      rm -rf $SYSTEM/app/CalculatorGooglePrebuilt
      remove_line $TMP/addon.d.sh "app/CalculatorGooglePrebuilt/CalculatorGooglePrebuilt.apk"
    fi
    if [ "$rm_sounds" = "1" ]; then
      ui_print "- Removing GoogleSoundPicker"
      sleep 0.3
      rm -rf $SYSTEM/app/SoundPickerPrebuilt
      remove_line $TMP/addon.d.sh "app/SoundPickerPrebuilt/SoundPickerPrebuilt.apk"
    fi
    if [ "$rm_markup" = "1" ]; then
      ui_print "- Removing GoogleMarkup"
      sleep 0.3
      rm -rf $SYSTEM/app/MarkupGoogle
      rm -rf $SYSTEM/app/lib64/libsketchology_native.so
      remove_line $TMP/addon.d.sh "app/MarkupGoogle/MarkupGoogle.apk"
      remove_line $TMP/addon.d.sh "lib64/libsketchology_native.so"
      remove_line $TMP/addon.d.sh "app/MarkupGoogle/lib/arm64/libsketchology_native.so"
    fi
    if [ "$rm_wallpapers" = "1" ]; then
      ui_print "- Removing GoogleWallpaperPicker"
      sleep 0.3
      rm -rf $SYSTEM/priv-app/WallpaperPickerGooglePrebuilt
      remove_line $TMP/addon.d.sh "priv-app/WallpaperPickerGooglePrebuilt/WallpaperPickerGooglePrebuilt.apk"
    fi
    if [ ! "$keep_snap" = "Y" ]; then
      rm -rf $SYSTEM/priv-app/Snap
      rm -rf $SYSTEM/priv-app/snap
      rm -rf $SYSTEM/app/SimpleCamera
      rm -rf $SYSTEM/priv-app/SimpleCamera
      if [ "$rom_sdk" -gt "28" ]; then
        rm -rf $SYSTEM/product/priv-app/Snap
        rm -rf $SYSTEM/product/priv-app/snap
        rm -rf $SYSTEM/product/app/SimpleCamera
        rm -rf $SYSTEM/product/priv-app/SimpleCamera
      fi
    else
      remove_line $TMP/addon.d.sh "priv-app/Snap"
      remove_line $TMP/addon.d.sh "app/SimpleCamera"
      remove_line $TMP/addon.d.sh "priv-app/SimpleCamera"
    fi
  else
    rm -rf $SYSTEM/priv-app/Snap
    rm -rf $SYSTEM/priv-app/snap
    rm -rf $SYSTEM/app/SimpleCamera
    rm -rf $SYSTEM/priv-app/SimpleCamera
    if [ "$rom_sdk" -gt "28" ]; then
      rm -rf $SYSTEM/product/priv-app/Snap
      rm -rf $SYSTEM/product/priv-app/snap
      rm -rf $SYSTEM/product/app/SimpleCamera
      rm -rf $SYSTEM/product/priv-app/SimpleCamera
    fi
  fi
}

# Unzip & install core gapps files
echo "  " >> $flame_log
echo "- Unzipping Core GApps Files" >> $flame_log
unzip_core >> $flame_log
echo "  " >> $flame_log
echo "- Installing Core GApps Files" >> $flame_log
install_core >> $flame_log
echo "  " >> $flame_log

# Unzip & install gapps files
echo "  " >> $flame_log
echo "- Unzipping GApps Files" >> $flame_log
unzip_gapps >> $flame_log
echo "  " >> $flame_log

# Install gapps files
echo "- Installing GApps Files" >> $flame_log
echo "  " >> $flame_log
install_gapps >> $flame_log

# Run debloater
echo "- Debloating Device" >> $flame_log
echo "  " >> $flame_log
if [ "$flame_edition" = "basic" ]; then
  run_basic_config
elif [ "$flame_edition" = "full" ]; then
  run_full_config
fi

echo "          Installation Finished " >> $flame_log
echo --------------------------------------------------------------- >> $flame_log

sleep 0.5
set_progress 0.80
ui_print " "
ui_print "- Performing misc tasks"
# Delete provision and lineage setupwizard if Google SetupWizard is available
if [ -e $SYSTEM/priv-app/SetupWizard/SetupWizard.apk ]; then
  rm -rf $SYSTEM/priv-app/Provision
  rm -rf $SYSTEM/priv-app/provision
  rm -rf $SYSTEM/priv-app/LineageSetupWizard
  if [ "$rom_sdk" -gt "28" ]; then
    rm -rf $SYSTEM/product/priv-app/Provision
    rm -rf $SYSTEM/product/priv-app/provision
    rm -rf $SYSTEM/product/priv-app/LineageSetupWizard
  fi
  [ "$rom_sdk" -gt "29" ] && rm -rf $SYSTEM/system_ext/priv-app/Provision
fi

# Delete AOSP Dialer if Google Dialer is available
if [ -e $SYSTEM/priv-app/GoogleDialer/GoogleDialer.apk ]; then
  rm -rf $SYSTEM/priv-app/Dialer
  if [ "$rom_sdk" -gt "28" ]; then
    rm -rf $SYSTEM/product/priv-app/Dialer
  fi
fi

# Install addon.d script
if [ -d "$SYSTEM/addon.d" ]; then
  rm -rf $SYSTEM/addon.d/69_flame.sh
  cp -f $TMP/addon.d.sh $SYSTEM/addon.d/69_flame.sh
  chmod 0755 $SYSTEM/addon.d/69_flame.sh
fi

# Install flame.prop
rm -rf $SYSTEM/etc/flame.prop
cp -f $TMP/flame.prop $SYSTEM/etc/flame.prop
chmod 0644 $SYSTEM/etc/flame.prop

# Set Google Dialer as default phone app if available
if [ -e $SYSTEM/priv-app/GoogleDialer/GoogleDialer.apk ]; then
  # set Google Dialer as default; based on the work of osm0sis @ xda-developers
  setver="122"  # lowest version in MM, tagged at 6.0.0
  setsec="/data/system/users/0/settings_secure.xml"
  if [ -f "$setsec" ]; then
    if grep -q 'dialer_default_application' "$setsec"; then
      if ! grep -q 'dialer_default_application" value="com.google.android.dialer' "$setsec"; then
        curentry="$(grep -o 'dialer_default_application" value=.*$' "$setsec")"
        newentry='dialer_default_application" value="com.google.android.dialer" package="android" />\r'
        sed -i "s;${curentry};${newentry};" "$setsec"
      fi
    else
      max="0"
      for i in $(grep -o 'id=.*$' "$setsec" | cut -d '"' -f 2); do
        test "$i" -gt "$max" && max="$i"
      done
      entry='<setting id="'"$((max + 1))"'" name="dialer_default_application" value="com.google.android.dialer" package="android" />\r'
      sed -i "/<settings version=\"/a\ \ ${entry}" "$setsec"
    fi
  else
    if [ ! -d "/data/system/users/0" ]; then
      install -d "/data/system/users/0"
      chown -R 1000:1000 "/data/system"
      chmod -R 775 "/data/system"
      chmod 700 "/data/system/users/0"
    fi
    { echo -e "<?xml version='1.0' encoding='UTF-8' standalone='yes' ?>\r";
    echo -e '<settings version="'$setver'">\r';
    echo -e '  <setting id="1" name="dialer_default_application" value="com.google.android.dialer" package="android" />\r';
    echo -e '</settings>'; } > "$setsec"
  fi
  chown 1000:1000 "$setsec"
  chmod 600 "$setsec"
fi

exit_all;
